﻿namespace Crack
{
    using System;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.Runtime.InteropServices;

    public class ScreenCapture
    {
        private int _WindowBottomPix;
        private int _WindowLeftPix;
        private int _WindowRightPix;
        private int _WindowTopPix;

        public Image CaptureScreen()
        {
            return this.CaptureWindow(User32.GetDesktopWindow());
        }

        public void CaptureScreenToFile(string filename, ImageFormat format)
        {
            this.CaptureScreen().Save(filename, format);
        }

        public Image CaptureWindow(IntPtr handle)
        {
            return this.CaptureWindow(handle, 0x129, 0xb8);
        }

        public Image CaptureWindow(IntPtr handle, int x, int y)
        {
            IntPtr windowDC = User32.GetWindowDC(handle);
            User32.RECT rect = new User32.RECT();
            User32.GetWindowRect(handle, ref rect);
            int nWidth = rect.right - rect.left;
            int nHeight = rect.bottom - rect.top;
            this._WindowLeftPix = rect.left;
            this._WindowRightPix = rect.right;
            this._WindowTopPix = rect.top;
            this._WindowBottomPix = rect.bottom;
            IntPtr hDC = GDI32.CreateCompatibleDC(windowDC);
            IntPtr hObject = GDI32.CreateCompatibleBitmap(windowDC, nWidth, nHeight);
            IntPtr ptr4 = GDI32.SelectObject(hDC, hObject);
            GDI32.BitBlt(hDC, 0, 0, nWidth, x, windowDC, 0, y, 0xcc0020);
            GDI32.SelectObject(hDC, ptr4);
            GDI32.DeleteDC(hDC);
            User32.ReleaseDC(handle, windowDC);
            Image image = Image.FromHbitmap(hObject);
            GDI32.DeleteObject(hObject);
            return image;
        }

        public void CaptureWindowToFile(IntPtr handle, string filename, ImageFormat format)
        {
            this.CaptureWindow(handle).Save(filename, format);
        }

        public void LocateWindow(IntPtr handle)
        {
            User32.GetWindowDC(handle);
            User32.RECT rect = new User32.RECT();
            User32.GetWindowRect(handle, ref rect);
            this._WindowLeftPix = rect.left;
            this._WindowRightPix = rect.right;
            this._WindowTopPix = rect.top;
            this._WindowBottomPix = rect.bottom;
        }

        public int WindowBottomPix
        {
            get
            {
                return this._WindowBottomPix;
            }
            set
            {
                this._WindowBottomPix = value;
            }
        }

        public int WindowLeftPix
        {
            get
            {
                return this._WindowLeftPix;
            }
            set
            {
                this._WindowLeftPix = value;
            }
        }

        public int WindowRightPix
        {
            get
            {
                return this._WindowRightPix;
            }
            set
            {
                this._WindowRightPix = value;
            }
        }

        public int WindowTopPix
        {
            get
            {
                return this._WindowTopPix;
            }
            set
            {
                this._WindowTopPix = value;
            }
        }

        private class GDI32
        {
            public const int SRCCOPY = 0xcc0020;

            [DllImport("gdi32.dll")]
            public static extern bool BitBlt(IntPtr hObject, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hObjectSource, int nXSrc, int nYSrc, int dwRop);
            [DllImport("gdi32.dll")]
            public static extern IntPtr CreateCompatibleBitmap(IntPtr hDC, int nWidth, int nHeight);
            [DllImport("gdi32.dll")]
            public static extern IntPtr CreateCompatibleDC(IntPtr hDC);
            [DllImport("gdi32.dll")]
            public static extern bool DeleteDC(IntPtr hDC);
            [DllImport("gdi32.dll")]
            public static extern bool DeleteObject(IntPtr hObject);
            [DllImport("gdi32.dll")]
            public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);
        }

        public class User32
        {
            [DllImport("user32.dll")]
            public static extern IntPtr GetDesktopWindow();
            [DllImport("user32.dll")]
            public static extern IntPtr GetWindowDC(IntPtr hWnd);
            [DllImport("user32.dll")]
            public static extern IntPtr GetWindowRect(IntPtr hWnd, ref RECT rect);
            [DllImport("user32.dll")]
            public static extern IntPtr ReleaseDC(IntPtr hWnd, IntPtr hDC);
            [DllImport("user32.dll")]
            public static extern IntPtr WindowFromPoint(POINT pt);

            [StructLayout(LayoutKind.Sequential)]
            public struct POINT
            {
                public int x;
                public int y;
            }

            [StructLayout(LayoutKind.Sequential)]
            public struct RECT
            {
                public int left;
                public int top;
                public int right;
                public int bottom;
            }
        }
    }
}

